creator_lst_good = [
    "3-Heights(TM) PDF Security Shell 4.8.25.2 (http://www.pdf-tools.com)",
    "PDFium",
    "pdfmake",
    "Style Report",
    "Crystal Reports",
    "JasperReports",
    "wkhtmltopdf 0.12.6",
    "wkhtmltopdf",
    "Ricoh Americas Corporation",
    "BIRT Report Engine .",
    "BIRT Engine",
    "FreeFlow VI eCompose 15.0.3.0",
    "iECCM, using PDF Engine.",
    "STANDARD CHARTERED BANK",
    "PDFKit",
    "Octagon 5.0",
    "alrajhi bank",
    "GMC Software AG",
    "HP Exstream Version",
    "OpenText Exstream Version",
    "vDOX",
    "Apache FOP Version ComplexScripts",
    "Quadient Group AG~Inspire"]

creator_lst_bad = [
    "Chrome",
    "Mozilla",
    "Microsoft",
    "Safari",
    "Samsung",
    "Acrobat",
    "macOS",
    "jsPDF 1.5.3",
    "Microsoft: Print To PDF"
    ]

producer_lst_good = [
    "Apache FOP Version ComplexScripts",
    "3-Heights(TM) PDF Security Shell 4.8.25.2 (http://www.pdf-tools.com)",
    "iText",
    "iText 5.0.2 (c) 1T3XT BVBA",
    "iText 2.0.8 (by lowagie.com)",
    "iECCM, using PDF Engine.",
    "pdfmake",
    "Ricoh Americas Corporation",
    "Powered By Crystal",
    "Style Report",
    "Qt 4.8.7",
    "PDFium",
    "pdfium",
    "Quadient Group AG~Inspire"
    ]

producer_lst_bad = [
    "ilovePDF",
    "iOS",
    "macOS Version 13.3.1 (a) (Build 22E772610a) Quartz PDFContext",
    "OpenPDF 1.3.26",
    "PSPDFKit",
    "Tango+ PDF Writer",
    "excel",
    "canva",
    "word",
    "jsPDF 1.5.3",
    "Microsoft: Print To PDF"
    ]